import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.URL;
import java.net.URLConnection;
import java.net.URLEncoder;
import java.util.Map;
import com.google.gson.Gson;

/**
 * This class provides a method to do http-post-requests using the java URL class
 * @author Newsletter2Go-Dev-Team. Questions? => dev@newsletter2go.de
 *
 */
public class Post_Request {

	public ApiResult doRequest(String host, String key, Map<SendEmail.params, String> req_params, Map<SendEmail.params, String> opt_params ) {

		try {
			// Construct the post data
			StringBuilder data = new StringBuilder();
			//add auth info:
		    data.append(URLEncoder.encode("key", "UTF-8") + "=" + URLEncoder.encode(key, "UTF-8"));
		    data.append("&");
			//add required params:
			for (Map.Entry<SendEmail.params, String> entry : req_params.entrySet()){
				//make param keys toLowerCase, cause API requires
			    data.append(URLEncoder.encode(entry.getKey().toString().toLowerCase(), "UTF-8") + "=" + URLEncoder.encode(entry.getValue().toString(), "UTF-8"));
			    data.append("&");			    	
			}
			//add optional params, if available
			for (Map.Entry<SendEmail.params, String> entry : opt_params.entrySet()){
				if(entry.getValue() != null){
					//make param keys toLowerCase, cause API requires
				    data.append(URLEncoder.encode(entry.getKey().toString().toLowerCase(), "UTF-8") + "=" + URLEncoder.encode(entry.getValue().toString(), "UTF-8"));
				    data.append("&");			 
				}
			}
			//trim last ampersand
			data.deleteCharAt(data.length()-1);
			
			//System.out.println(data.toString());
			
			// Send data via post
			URL url = new URL(host);
			URLConnection conn = url.openConnection();
			conn.setDoOutput(true);
			OutputStreamWriter wr = new OutputStreamWriter(conn.getOutputStream());
			wr.write(data.toString());
			wr.flush();
			// Get the response:
			BufferedReader rd = new BufferedReader(new InputStreamReader(conn.getInputStream()),Integer.MAX_VALUE);
			String json="";
			String line;
			while ((line = rd.readLine()) != null) {
				// get JSON result from Newsletter2Go API:
				json += line;				
			}
			//decode json to an ApiResult object
			Gson gson = new Gson();
			ApiResult res = gson.fromJson(json, ApiResult.class);
			//close streams:
			wr.close();
			rd.close();
			return res;
		} catch (Exception e) {
			e.printStackTrace();
			//TODO: you shouldnt do this in a productive environment:
			return null;
		}

	}

}
