import java.util.HashMap;
import java.util.Map;

/**
 * This is an example class, showing how to send an email using the Newsletter2Go API.
 * Note: This is just an example. There are many other ways to do this.
 * @author Newsletter2Go-Dev-Team. Questions? => dev@newsletter2go.de
 *
 */
public class SendEmail{

	/**
	 * required and optional parameters for sending an email
	 */
	public enum params{
		TO,
		FROM,
		SUBJECT,
		HTML,
		TEXT,
		ID,
		DEBUG,
		REF,
		DATE,
		REPLY
	}
	
	
	private String api_url ="https://www.newsletter2go.de/de/api/send/email/";	//"https://www.newsletter2go.de/de/api/send/email/"
	
	private String key;

	
	/**
	 * required params
	 */
	private Map<params, String> req_params = new HashMap<params, String>();	
	
	/**
	 * optional params
	 */
	private Map<params, String> opt_params = new HashMap<params, String>();
		
	/**
	 * Constructor
	 * @param user your username at Newsletter2Go
	 * @param key your api key
	 * @param secret your api secret
	 */
	public SendEmail(String key){
		//TODO: test if arguments != ""
		this.key = key;
		//initialize param maps:
		req_params.put(params.TO, null);
		req_params.put(params.FROM, null);
		req_params.put(params.SUBJECT, null);
		req_params.put(params.HTML, null);
		req_params.put(params.TEXT, null);
		
		opt_params.put(params.ID, null);
		opt_params.put(params.DEBUG, null);
		opt_params.put(params.REF, null);
		opt_params.put(params.DATE, null);
		opt_params.put(params.REPLY, null);
	}
	
	/**
	 * Method to add a param to this request
	 * @param param name of the paramter
	 * @param value value of the parameter
	 * @throws IllegalArgumentException if the param is not allowed
	 */
	public void addParam(params param, String value) throws IllegalArgumentException{
		if(param == params.TO || param == params.FROM || param == params.SUBJECT || param == params.HTML || param == params.TEXT ) {
			req_params.put(param, value);
		}
		else{
			if(param == params.ID || param == params.DEBUG || param == params.REF || param == params.DATE || param == params.REPLY) {
				opt_params.put(param, value);
			}
			else{
				//bad param
				throw new IllegalArgumentException(param.toString() + " is not known.");
			}
		}
	}
	
	/**
	 * Sends this email via POST-request.
	 * @return the result from Newsletter2Go API encapsulated in a result object ApiResult
	 * @throws MissingParamException if at least 1 required param is missing
	 */
	public ApiResult send() throws MissingParamException{
		//check if all required params are available:
		for (Map.Entry<params, String> entry : this.req_params.entrySet()){
		    if(entry.getValue() == null || entry.getValue().equals(""))
		    	throw new MissingParamException(entry.getKey() + " is missing.");
		}
		//else do the post request:
		return new Post_Request().doRequest(api_url,this.key, req_params, opt_params);
	}
	
}
